@@ -1,4 +1,4 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
+require_relative "spec_helper"
 
 describe "Simple Dataset operations" do
   before do
@@ -15,8 +15,8 @@ describe "Simple Dataset operations" do
   end
 
   it "should support sequential primary keys" do
-    @ds << {:number=>20}
-    @ds << {:number=>30}
+    @ds.insert(:number=>20)
+    @ds.insert(:number=>30)
     @ds.order(:number).all.must_equal [
       {:id => 1, :number=>10},
       {:id => 2, :number=>20},
@@ -28,8 +28,8 @@ describe "Simple Dataset operations" do
       primary_key :id, :type=>:Bignum
       Integer :number
     end
-    @ds << {:number=>20}
-    @ds << {:number=>30}
+    @ds.insert(:number=>20)
+    @ds.insert(:number=>30)
     @ds.order(:number).all.must_equal [{:id => 1, :number=>20}, {:id => 2, :number=>30}]   
   end 
 
@@ -49,18 +49,12 @@ describe "Simple Dataset operations" do
     @ds.filter(:id=>2).first[:number].must_equal 20
   end
 
-  it "should have insert_multiple return primary key values" do
-    @ds.extension(:sequel_3_dataset_methods).insert_multiple([{:number=>20}, {:number=>30}]).must_equal [2, 3]
-    @ds.filter(:id=>2).get(:number).must_equal 20
-    @ds.filter(:id=>3).get(:number).must_equal 30
-  end
-
   it "should join correctly" do
     @ds.join(Sequel[:items].as(:b), :id=>:id).select_all(:items).all.must_equal [{:id=>1, :number=>10}]
   end
 
   it "should handle LATERAL subqueries correctly" do
-    @ds << {:number=>20}
+    @ds.insert(:number=>20)
     @ds.from(Sequel[:items].as(:i), @ds.where(Sequel[:items][:number]=>Sequel[:i][:number]).lateral).select_order_map([Sequel[:i][:number].as(:n), Sequel[:t1][:number]]).must_equal [[10, 10], [20, 20]]
     @ds.from(Sequel[:items].as(:i)).cross_join(@ds.where(Sequel[:items][:number]=>Sequel[:i][:number]).lateral).select_order_map([Sequel[:i][:number].as(:n), Sequel[:t1][:number]]).must_equal [[10, 10], [20, 20]]
     @ds.from(Sequel[:items].as(:i)).join(@ds.where(Sequel[:items][:number]=>Sequel[:i][:number]).lateral, 1=>1).select_order_map([Sequel[:i][:number].as(:n), Sequel[:t1][:number]]).must_equal [[10, 10], [20, 20]]
@@ -108,7 +102,7 @@ describe "Simple Dataset operations" do
     @ds.all.must_equal [{:id=>1, :number=>11}]
   end
   
-  cspecify "should have update return the number of matched rows", [:do, :mysql] do
+  it "should have update return the number of matched rows" do
     @ds.update(:number=>:number).must_equal 1
     @ds.filter(:id=>1).update(:number=>:number).must_equal 1
     @ds.filter(:id=>2).update(:number=>:number).must_equal 0
@@ -171,7 +165,7 @@ describe "Simple Dataset operations" do
     @ds.all.must_equal [{:id=>1, :number=>10}]
   end
 
-  cspecify "should skip locked rows correctly", [:do] do
+  it "should skip locked rows correctly" do
     @ds.insert(:number=>10)
     q1 = Queue.new
     q2 = Queue.new
@@ -189,6 +183,24 @@ describe "Simple Dataset operations" do
     end
   end if DB.dataset.supports_skip_locked?
   
+  it "should raise error instead of waiting for rows correctly" do
+    @ds.insert(:number=>10)
+    q1 = Queue.new
+    q2 = Queue.new
+    ds = @ds.order(:id).for_update.nowait
+    begin
+      t = Thread.new{@db.transaction(:isolation=>:committed){q2.push(ds.get(:id)); q1.pop}}
+      q2.pop.must_equal 1
+      # Some databases do row level locking, others do page level locking
+      proc{@db.transaction(:isolation=>:committed){ds.get(:id)}}.must_raise Sequel::DatabaseLockTimeout
+    ensure
+      q1.push(nil)
+      t.join
+      # Keep only one active connection, as some other specs expect that
+      @db.disconnect
+    end
+  end if DB.dataset.supports_nowait?
+  
   it "should raise exception if raising on duplication columns" do
     proc{@ds.select_map([:id, :id])}.must_raise Sequel::DuplicateColumnError
   end if DB.opts[:on_duplicate_columns] == :raise
@@ -389,9 +401,9 @@ describe Sequel::Dataset do
 
   it "should return the correct record count" do
     @d.count.must_equal 0
-    @d << {:name => 'abc', :value => 123}
-    @d << {:name => 'abc', :value => 456}
-    @d << {:name => 'def', :value => nil}
+    @d.insert(:name => 'abc', :value => 123)
+    @d.insert(:name => 'abc', :value => 456)
+    @d.insert(:name => 'def', :value => nil)
     5.times do
       @d.count.must_equal 3
       @d.count(:name).must_equal 3
@@ -400,14 +412,14 @@ describe Sequel::Dataset do
   end
 
   it "should handle functions with identifier names correctly" do
-    @d << {:name => 'abc', :value => 6}
+    @d.insert(:name => 'abc', :value => 6)
     @d.get{sum.function(:value)}.must_equal 6
   end
 
   it "should handle aggregate methods on limited datasets correctly" do
-    @d << {:name => 'abc', :value => 6}
-    @d << {:name => 'bcd', :value => 12}
-    @d << {:name => 'def', :value => 18}
+    @d.insert(:name => 'abc', :value => 6)
+    @d.insert(:name => 'bcd', :value => 12)
+    @d.insert(:name => 'def', :value => 18)
     @d = @d.order(:name).limit(2)
     @d.count.must_equal 2
     @d.avg(:value).to_i.must_equal 9
@@ -415,14 +427,14 @@ describe Sequel::Dataset do
     @d.reverse.min(:value).to_i.must_equal 12
     @d.max(:value).to_i.must_equal 12
     @d.sum(:value).to_i.must_equal 18
-    @d.interval(:value).to_i.must_equal 6
+    @d.extension(:sequel_4_dataset_methods).interval(:value).to_i.must_equal 6
   end
 
   it "should return the correct records" do
     @d.to_a.must_equal []
-    @d << {:name => 'abc', :value => 123}
-    @d << {:name => 'abc', :value => 456}
-    @d << {:name => 'def', :value => 789}
+    @d.insert(:name => 'abc', :value => 123)
+    @d.insert(:name => 'abc', :value => 456)
+    @d.insert(:name => 'def', :value => 789)
 
     @d.order(:value).to_a.must_equal [
       {:name => 'abc', :value => 123},
@@ -432,27 +444,27 @@ describe Sequel::Dataset do
   end
 
   it "should update records correctly" do
-    @d << {:name => 'abc', :value => 123}
-    @d << {:name => 'abc', :value => 456}
-    @d << {:name => 'def', :value => 789}
+    @d.insert(:name => 'abc', :value => 123)
+    @d.insert(:name => 'abc', :value => 456)
+    @d.insert(:name => 'def', :value => 789)
     @d.filter(:name => 'abc').update(:value => 530)
     @d[:name => 'def'][:value].must_equal 789
     @d.filter(:value => 530).count.must_equal 2
   end
 
   it "should delete records correctly" do
-    @d << {:name => 'abc', :value => 123}
-    @d << {:name => 'abc', :value => 456}
-    @d << {:name => 'def', :value => 789}
+    @d.insert(:name => 'abc', :value => 123)
+    @d.insert(:name => 'abc', :value => 456)
+    @d.insert(:name => 'def', :value => 789)
     @d.filter(:name => 'abc').delete
     @d.count.must_equal 1
     @d.first[:name].must_equal 'def'
   end
   
   it "should be able to truncate the table" do
-    @d << {:name => 'abc', :value => 123}
-    @d << {:name => 'abc', :value => 456}
-    @d << {:name => 'def', :value => 789}
+    @d.insert(:name => 'abc', :value => 123)
+    @d.insert(:name => 'abc', :value => 456)
+    @d.insert(:name => 'def', :value => 789)
     @d.count.must_equal 3
     @d.truncate.must_be_nil
     @d.count.must_equal 0
@@ -517,9 +529,9 @@ describe Sequel::Dataset do
       Integer :value
     end 
     @d = DB[:items]
-    @d << {:value => 123}
-    @d << {:value => 456}
-    @d << {:value => 789}
+    @d.insert(:value => 123)
+    @d.insert(:value => 456)
+    @d.insert(:value => 789)
   end 
   after do
     DB.drop_table?(:items)
@@ -616,6 +628,10 @@ describe "Dataset UNION, EXCEPT, and INTERSECT" do
     DB.drop_table?(:i1, :i2, :i3)
   end
   
+  it "should give the correct results for UNION with an existing order" do
+    @ds1.order(:number).union(@ds2).order(:number).map{|x| x[:number].to_s}.must_equal %w'10 20 30'
+  end
+
   it "should give the correct results for simple UNION, EXCEPT, and INTERSECT" do
     @ds1.union(@ds2).order(:number).map{|x| x[:number].to_s}.must_equal %w'10 20 30'
     if @ds1.supports_intersect_except?
@@ -624,7 +640,10 @@ describe "Dataset UNION, EXCEPT, and INTERSECT" do
     end
   end
   
-  cspecify "should give the correct results for UNION, EXCEPT, and INTERSECT when used with ordering and limits", :mssql do
+  it "should give the correct results for UNION, EXCEPT, and INTERSECT when used with ordering and limits and offsets" do
+    [%w'10 30', %w'10 20 30'].must_include @ds1.limit(1).union(@ds2).order(:number).map{|x| x[:number].to_s}
+    [%w'10 30', %w'10 20 30'].must_include @ds1.offset(1).union(@ds2).order(:number).map{|x| x[:number].to_s}
+
     @ds1.insert(:number=>8)
     @ds2.insert(:number=>9)
     @ds1.insert(:number=>38)
@@ -635,15 +654,31 @@ describe "Dataset UNION, EXCEPT, and INTERSECT" do
 
     @ds1.reverse_order(:number).limit(1).union(@ds2).order(:number).map{|x| x[:number].to_s}.must_equal %w'9 10 30 38 39'
     @ds2.reverse_order(:number).limit(1).union(@ds1).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 10 20 38 39'
+    @ds1.reverse_order(:number).limit(1, 1).union(@ds2).order(:number).map{|x| x[:number].to_s}.must_equal %w'9 10 20 30 39'
+    @ds2.reverse_order(:number).limit(1, 1).union(@ds1).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 10 20 30 38'
+    @ds1.reverse_order(:number).offset(1).union(@ds2).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 9 10 20 30 39'
+    @ds2.reverse_order(:number).offset(1).union(@ds1).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 9 10 20 30 38'
 
     @ds1.union(@ds2.order(:number).limit(1)).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 9 10 20 38'
     @ds2.union(@ds1.order(:number).limit(1)).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 9 10 30 39'
+    @ds1.union(@ds2.order(:number).limit(1, 1)).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 10 20 38'
+    @ds2.union(@ds1.order(:number).limit(1, 1)).order(:number).map{|x| x[:number].to_s}.must_equal %w'9 10 30 39'
+    @ds1.union(@ds2.order(:number).offset(1)).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 10 20 30 38 39'
+    @ds2.union(@ds1.order(:number).offset(1)).order(:number).map{|x| x[:number].to_s}.must_equal %w'9 10 20 30 38 39'
 
     @ds1.union(@ds2).limit(2).order(:number).map{|x| x[:number].to_s}.must_equal %w'8 9'
     @ds2.union(@ds1).reverse_order(:number).limit(2).map{|x| x[:number].to_s}.must_equal %w'39 38'
+    @ds1.union(@ds2).limit(2, 1).order(:number).map{|x| x[:number].to_s}.must_equal %w'9 10'
+    @ds2.union(@ds1).reverse_order(:number).limit(2, 1).map{|x| x[:number].to_s}.must_equal %w'38 30'
+    @ds1.union(@ds2).offset(1).order(:number).map{|x| x[:number].to_s}.must_equal %w'9 10 20 30 38 39'
+    @ds2.union(@ds1).reverse_order(:number).offset(1).map{|x| x[:number].to_s}.must_equal %w'38 30 20 10 9 8'
 
     @ds1.reverse_order(:number).limit(2).union(@ds2.reverse_order(:number).limit(2)).order(:number).limit(3).map{|x| x[:number].to_s}.must_equal %w'20 30 38'
     @ds2.order(:number).limit(2).union(@ds1.order(:number).limit(2)).reverse_order(:number).limit(3).map{|x| x[:number].to_s}.must_equal %w'10 9 8'
+    @ds1.reverse_order(:number).limit(2, 1).union(@ds2.reverse_order(:number).limit(2, 1)).order(:number).limit(3, 1).map{|x| x[:number].to_s}.must_equal %w'20 30'
+    @ds2.order(:number).limit(2, 1).union(@ds1.order(:number).limit(2, 1)).reverse_order(:number).limit(3, 1).map{|x| x[:number].to_s}.must_equal %w'20 10'
+    @ds1.reverse_order(:number).offset(1).union(@ds2.reverse_order(:number).offset(1)).order(:number).offset(1).map{|x| x[:number].to_s}.must_equal %w'9 10 20 30'
+    @ds2.order(:number).offset(1).union(@ds1.order(:number).offset(1)).reverse_order(:number).offset(1).map{|x| x[:number].to_s}.must_equal %w'38 30 20 10'
   end
 
   it "should give the correct results for compound UNION, EXCEPT, and INTERSECT" do
@@ -732,8 +767,8 @@ if DB.dataset.supports_cte?
   end
 end
 
-if DB.dataset.supports_cte?(:update) # Assume INSERT and DELETE support as well
-  describe "Common Table Expressions in INSERT/UPDATE/DELETE" do
+if DB.dataset.supports_cte?(:insert) || DB.dataset.supports_cte?(:update) || DB.dataset.supports_cte?(:delete)
+  describe "Common Table Expressions" do
     before do
       @db = DB
       @db.create_table!(:i1){Integer :id}
@@ -746,13 +781,26 @@ if DB.dataset.supports_cte?(:update) # Assume INSERT and DELETE support as well
       @db.drop_table?(:i1)
     end
     
-    it "should give correct results for WITH" do
+    it "should give correct results for WITH in insert" do
       @ds2.insert(@db[:t])
       @ds.select_order_map(:id).must_equal [1, 1, 2, 2]
+    end if DB.dataset.supports_cte?(:insert)
+
+    it "should give correct results for WITH in update" do
       @ds2.filter(:id=>@db[:t].select{max(id)}).update(:id=>Sequel.+(:id, 1))
-      @ds.select_order_map(:id).must_equal [1, 1, 3, 3]
+      @ds.select_order_map(:id).must_equal [1, 3]
+    end if DB.dataset.supports_cte?(:update)
+
+    it "should give correct results for WITH in delete" do
       @ds2.filter(:id=>@db[:t].select{max(id)}).delete
-      @ds.select_order_map(:id).must_equal [1, 1]
+      @ds.select_order_map(:id).must_equal [1]
+    end if DB.dataset.supports_cte?(:delete)
+
+    it "should support a subselect in an subquery used for INSERT" do
+      @db.transaction(:rollback=>:always) do
+        @ds.insert([:id], @db[:foo].with(:foo, @ds.select{(id + 10).as(:id)}))
+        @ds.select_order_map(:id).must_equal [1,2,11,12]
+      end
     end
   end
 end
@@ -843,9 +891,9 @@ if DB.dataset.supports_window_functions?
     end
       
     it "should give correct results for ranking window functions with orders" do
-      @ds.select(:id){rank{}.over(:partition=>:group_id, :order=>:id).as(:rank)}.all.
+      @ds.select(:id){rank.function.over(:partition=>:group_id, :order=>:id).as(:rank)}.all.
         must_equal [{:rank=>1, :id=>1}, {:rank=>2, :id=>2}, {:rank=>3, :id=>3}, {:rank=>1, :id=>4}, {:rank=>2, :id=>5}, {:rank=>3, :id=>6}]
-      @ds.select(:id){rank{}.over(:order=>id).as(:rank)}.all.
+      @ds.select(:id){rank.function.over(:order=>id).as(:rank)}.all.
         must_equal [{:rank=>1, :id=>1}, {:rank=>2, :id=>2}, {:rank=>3, :id=>3}, {:rank=>4, :id=>4}, {:rank=>5, :id=>5}, {:rank=>6, :id=>6}]
     end
       
@@ -854,6 +902,8 @@ if DB.dataset.supports_window_functions?
         must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
       @ds.select(:id){sum(:amount).over(:order=>:id).as(:sum)}.all.
         must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>11111, :id=>5}, {:sum=>111111, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:partition=>:group_id, :order=>id.desc, :frame=>:rows).as(:sum)}.all.
+        must_equal [{:sum=>111, :id=>1}, {:sum=>110, :id=>2}, {:sum=>100, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>110000, :id=>5}, {:sum=>100000, :id=>6}]
     end
     
     it "should give correct results for aggregate window functions with frames" do
@@ -867,6 +917,71 @@ if DB.dataset.supports_window_functions?
       @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>:rows).as(:sum)}.all.
         must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>11111, :id=>5}, {:sum=>111111, :id=>6}]
     end
+
+    it "should give correct results for aggregate window functions with ranges" do
+      @ds.select(:id){sum(:amount).over(:partition=>:group_id, :order=>:id, :frame=>:range).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:partition=>:group_id, :order=>:group_id, :frame=>:range).as(:sum)}.all.
+        must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>111000, :id=>6}]
+    end if DB.dataset.supports_window_function_frame_option?(:range)
+
+    it "should give correct results for aggregate window functions with groups" do
+      @ds.select(:id){sum(:amount).over(:partition=>:group_id, :order=>:id, :frame=>:groups).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>:groups).as(:sum)}.all.
+        must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111111, :id=>4}, {:sum=>111111, :id=>5}, {:sum=>111111, :id=>6}]
+    end if DB.dataset.supports_window_function_frame_option?(:groups)
+
+    if DB.dataset.supports_window_function_frame_option?(:offset)
+      it "should give correct results for aggregate window functions with offsets for ROWS" do
+        @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>1}).as(:sum)}.all.
+          must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>110, :id=>3}, {:sum=>1100, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>110000, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>1, :end=>1}).as(:sum)}.all.
+          must_equal [{:sum=>11, :id=>1}, {:sum=>111, :id=>2}, {:sum=>1110, :id=>3}, {:sum=>11100, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>110000, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>2, :end=>[1, :preceding]}).as(:sum)}.all.
+          must_equal [{:sum=>nil, :id=>1}, {:sum=>1, :id=>2}, {:sum=>11, :id=>3}, {:sum=>110, :id=>4}, {:sum=>1100, :id=>5}, {:sum=>11000, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>[1, :following], :end=>2}).as(:sum)}.order(:id).all.
+          must_equal [{:sum=>110, :id=>1}, {:sum=>1100, :id=>2}, {:sum=>11000, :id=>3}, {:sum=>110000, :id=>4}, {:sum=>100000, :id=>5}, {:sum=>nil, :id=>6}]
+      end
+
+      cspecify "should give correct results for aggregate window functions with offsets for RANGES", :mssql, [proc{DB.server_version < 110000}, :postgres] do
+        @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:range, :start=>1}).as(:sum)}.all.
+          must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111111, :id=>4}, {:sum=>111111, :id=>5}, {:sum=>111111, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:range, :start=>0, :end=>1}).as(:sum)}.all.
+          must_equal [{:sum=>111111, :id=>1}, {:sum=>111111, :id=>2}, {:sum=>111111, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>111000, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:amount, :frame=>{:type=>:range, :start=>100, :end=>1000}).as(:sum)}.all.
+          must_equal [{:sum=>1111, :id=>1}, {:sum=>1111, :id=>2}, {:sum=>1111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>10000, :id=>5}, {:sum=>100000, :id=>6}]
+      end if DB.dataset.supports_window_function_frame_option?(:range)
+
+      it "should give correct results for aggregate window functions with offsets for GROUPS" do
+        @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:groups, :start=>1}).as(:sum)}.all.
+          must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111111, :id=>4}, {:sum=>111111, :id=>5}, {:sum=>111111, :id=>6}]
+        @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:groups, :start=>0, :end=>1}).as(:sum)}.all.
+          must_equal [{:sum=>111111, :id=>1}, {:sum=>111111, :id=>2}, {:sum=>111111, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>111000, :id=>6}]
+      end if DB.dataset.supports_window_function_frame_option?(:groups)
+    end
+
+    it "should give correct results for aggregate window functions with exclusions" do
+      @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>:preceding, :exclude=>:current}).as(:sum)}.all.
+        must_equal [{:sum=>nil, :id=>1}, {:sum=>1, :id=>2}, {:sum=>11, :id=>3}, {:sum=>111, :id=>4}, {:sum=>1111, :id=>5}, {:sum=>11111, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:rows, :start=>:preceding, :exclude=>:group}).as(:sum)}.all.
+        must_equal [{:sum=>nil, :id=>1}, {:sum=>nil, :id=>2}, {:sum=>nil, :id=>3}, {:sum=>111, :id=>4}, {:sum=>111, :id=>5}, {:sum=>111, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:order=>:group_id, :frame=>{:type=>:rows, :start=>:preceding, :exclude=>:ties}).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>10, :id=>2}, {:sum=>100, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>10111, :id=>5}, {:sum=>100111, :id=>6}]
+      @ds.select(:id){sum(:amount).over(:order=>:id, :frame=>{:type=>:rows, :start=>:preceding, :exclude=>:no_others}).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>11111, :id=>5}, {:sum=>111111, :id=>6}]
+    end if DB.dataset.supports_window_function_frame_option?(:exclude)
+
+    it "should give correct results for window functions" do
+      @ds.window(:win, :partition=>:group_id, :order=>:id).select(:id){sum(:amount).over(:window=>win, :frame=>:rows).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
+      @ds.window(:win, :partition=>:group_id).select(:id){sum(:amount).over(:window=>win, :order=>id, :frame=>:rows).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1000, :id=>4}, {:sum=>11000, :id=>5}, {:sum=>111000, :id=>6}]
+      @ds.window(:win, {}).select(:id){sum(:amount).over(:window=>:win, :order=>id, :frame=>:rows).as(:sum)}.all.
+        must_equal [{:sum=>1, :id=>1}, {:sum=>11, :id=>2}, {:sum=>111, :id=>3}, {:sum=>1111, :id=>4}, {:sum=>11111, :id=>5}, {:sum=>111111, :id=>6}]
+      @ds.window(:win, :partition=>:group_id).select(:id){sum(:amount).over(:window=>:win, :order=>id, :frame=>:all).as(:sum)}.all.
+        must_equal [{:sum=>111, :id=>1}, {:sum=>111, :id=>2}, {:sum=>111, :id=>3}, {:sum=>111000, :id=>4}, {:sum=>111000, :id=>5}, {:sum=>111000, :id=>6}]
+    end if DB.dataset.supports_window_clause?
   end
 end
 
@@ -904,13 +1019,13 @@ describe Sequel::SQL::Constants do
     (Time.now - @c[@ds.get(:t)]).must_be_close_to 0, 60
   end
 
-  cspecify "should have working CURRENT_TIMESTAMP", [:jdbc, :sqlite], [:swift] do
+  cspecify "should have working CURRENT_TIMESTAMP", [:jdbc, :sqlite] do
     @db.create_table!(:constants){DateTime :ts}
     @ds.insert(:ts=>Sequel::CURRENT_TIMESTAMP)
     (Time.now - @c[@ds.get(:ts)]).must_be_close_to 0, 60
   end
 
-  cspecify "should have working CURRENT_TIMESTAMP when used as a column default", [:jdbc, :sqlite], [:swift] do
+  cspecify "should have working CURRENT_TIMESTAMP when used as a column default", [:jdbc, :sqlite] do
     @db.create_table!(:constants){DateTime :ts, :default=>Sequel::CURRENT_TIMESTAMP}
     @ds.insert
     (Time.now - @c[@ds.get(:ts)]).must_be_close_to 0, 60
@@ -1032,12 +1147,6 @@ describe "Sequel::Dataset convenience methods" do
     @db.drop_table?(:a)
   end
   
-  it "#[]= should update matching rows" do
-    @ds.insert(20, 10)
-    @ds.extension(:sequel_3_dataset_methods)[:a=>20] = {:b=>30}
-    @ds.all.must_equal [{:a=>20, :b=>30}]
-  end
-  
   it "#empty? should return whether the dataset returns no rows" do
     @ds.empty?.must_equal true
     @ds.insert(20, 10)
@@ -1081,7 +1190,7 @@ describe "Sequel::Dataset convenience methods" do
   end
   
   it "#range should return the range between the maximum and minimum values" do
-    @ds = @ds.unordered
+    @ds = @ds.unordered.extension(:sequel_4_dataset_methods)
     @ds.insert(20, 10)
     @ds.insert(30, 10)
     @ds.range(:a).must_equal(20..30)
@@ -1089,7 +1198,7 @@ describe "Sequel::Dataset convenience methods" do
   end
   
   it "#interval should return the different between the maximum and minimum values" do
-    @ds = @ds.unordered
+    @ds = @ds.unordered.extension(:sequel_4_dataset_methods)
     @ds.insert(20, 10)
     @ds.insert(30, 10)
     @ds.interval(:a).to_i.must_equal 10
@@ -1124,10 +1233,10 @@ describe "Sequel::Dataset main SQL methods" do
     @ds.filter(Sequel.lit("b < ?", 15)).invert.all.must_equal [{:a=>20, :b=>30}]
   end
   
-  it "#and and #or should work correctly" do
+  it "#where and #or should work correctly" do
     @ds.insert(20, 30)
-    @ds.filter(:a=>20).and(:b=>30).all.must_equal [{:a=>20, :b=>30}]
-    @ds.filter(:a=>20).and(:b=>15).all.must_equal []
+    @ds.filter(:a=>20).where(:b=>30).all.must_equal [{:a=>20, :b=>30}]
+    @ds.filter(:a=>20).where(:b=>15).all.must_equal []
     @ds.filter(:a=>20).or(:b=>15).all.must_equal [{:a=>20, :b=>30}]
     @ds.filter(:a=>10).or(:b=>15).all.must_equal []
   end
@@ -1189,17 +1298,19 @@ describe "Sequel::Dataset convenience methods" do
     @ds.map([:a, :b]).must_equal [[1, 2], [5, 6]]
   end
   
-  it "should have working #to_hash" do
+  it "should have working #as_hash" do
     @ds.to_hash(:a).must_equal(1=>{:a=>1, :b=>2, :c=>3, :d=>4}, 5=>{:a=>5, :b=>6, :c=>7, :d=>8})
-    @ds.to_hash(:b).must_equal(2=>{:a=>1, :b=>2, :c=>3, :d=>4}, 6=>{:a=>5, :b=>6, :c=>7, :d=>8})
-    @ds.to_hash([:a, :b]).must_equal([1, 2]=>{:a=>1, :b=>2, :c=>3, :d=>4}, [5, 6]=>{:a=>5, :b=>6, :c=>7, :d=>8})
+    @ds.as_hash(:a).must_equal(1=>{:a=>1, :b=>2, :c=>3, :d=>4}, 5=>{:a=>5, :b=>6, :c=>7, :d=>8})
+    @ds.as_hash(:b).must_equal(2=>{:a=>1, :b=>2, :c=>3, :d=>4}, 6=>{:a=>5, :b=>6, :c=>7, :d=>8})
+    @ds.as_hash([:a, :b]).must_equal([1, 2]=>{:a=>1, :b=>2, :c=>3, :d=>4}, [5, 6]=>{:a=>5, :b=>6, :c=>7, :d=>8})
 
-    @ds.to_hash(:a, :b).must_equal(1=>2, 5=>6)
-    @ds.to_hash([:a, :c], :b).must_equal([1, 3]=>2, [5, 7]=>6)
-    @ds.to_hash(:a, [:b, :c]).must_equal(1=>[2, 3], 5=>[6, 7])
-    @ds.to_hash([:a, :c], [:b, :d]).must_equal([1, 3]=>[2, 4], [5, 7]=>[6, 8])
+    @ds.as_hash(:a, :b).must_equal(1=>2, 5=>6)
+    @ds.as_hash([:a, :c], :b).must_equal([1, 3]=>2, [5, 7]=>6)
+    @ds.as_hash(:a, [:b, :c]).must_equal(1=>[2, 3], 5=>[6, 7])
+    @ds.as_hash([:a, :c], [:b, :d]).must_equal([1, 3]=>[2, 4], [5, 7]=>[6, 8])
+    @ds.extension(:null_dataset).nullify.as_hash([:a, :c], [:b, :d]).must_equal({})
 
-    @ds.to_hash(:a, :b, :hash => (tmp = {})).must_be_same_as(tmp)
+    @ds.as_hash(:a, :b, :hash => (tmp = {})).must_be_same_as(tmp)
   end
 
   it "should have working #to_hash_groups" do
@@ -1213,6 +1324,7 @@ describe "Sequel::Dataset convenience methods" do
     ds.to_hash_groups([:a, :c], :d).must_equal([1, 3]=>[4, 9], [5, 7]=>[8])
     ds.to_hash_groups(:a, [:b, :d]).must_equal(1=>[[2, 4], [2, 9]], 5=>[[6, 8]])
     ds.to_hash_groups([:a, :c], [:b, :d]).must_equal([1, 3]=>[[2, 4], [2, 9]], [5, 7]=>[[6, 8]])
+    @ds.extension(:null_dataset).nullify.to_hash_groups([:a, :c], [:b, :d]).must_equal({})
 
     ds.to_hash_groups(:a, :d, :hash => (tmp = {})).must_be_same_as(tmp)
   end
@@ -1222,6 +1334,7 @@ describe "Sequel::Dataset convenience methods" do
     @ds.select_map(:b).must_equal [2, 6]
     @ds.select_map([:a]).must_equal [[1], [5]]
     @ds.select_map([:a, :b]).must_equal [[1, 2], [5, 6]]
+    @ds.extension(:null_dataset).nullify.select_map([:a, :b]).must_equal []
 
     @ds.select_map(Sequel[:a].as(:e)).must_equal [1, 5]
     @ds.select_map(Sequel[:b].as(:e)).must_equal [2, 6]
@@ -1239,6 +1352,7 @@ describe "Sequel::Dataset convenience methods" do
     @ds.select_order_map(Sequel.qualify(:a, :b).as(:e)).must_equal [2, 6]
     @ds.select_order_map([:a]).must_equal [[1], [5]]
     @ds.select_order_map([Sequel.desc(:a), :b]).must_equal [[5, 6], [1, 2]]
+    @ds.extension(:null_dataset).nullify.select_order_map(:a).must_equal []
 
     @ds.select_order_map(Sequel[:a].as(:e)).must_equal [1, 5]
     @ds.select_order_map(Sequel[:b].as(:e)).must_equal [2, 6]
@@ -1259,6 +1373,7 @@ describe "Sequel::Dataset convenience methods" do
     @ds.select_hash(:a, [:b, :c]).must_equal(1=>[2, 3], 5=>[6, 7])
     @ds.select_hash([:a, :c], [:b, :d]).must_equal([1, 3]=>[2, 4], [5, 7]=>[6, 8])
     @ds.select_hash(:a, :b, :hash => (tmp = {})).must_be_same_as(tmp)
+    @ds.extension(:null_dataset).nullify.select_hash(:a, :b).must_equal({})
   end
 
   it "should have working #select_hash_groups" do
@@ -1273,6 +1388,7 @@ describe "Sequel::Dataset convenience methods" do
     ds.select_hash_groups(:a, [:b, :d]).must_equal(1=>[[2, 4], [2, 9]], 5=>[[6, 8]])
     ds.select_hash_groups([:a, :c], [:b, :d]).must_equal([1, 3]=>[[2, 4], [2, 9]], [5, 7]=>[[6, 8]])
     ds.select_hash_groups(:a, :d, :hash => (tmp = {})).must_be_same_as(tmp)
+    @ds.extension(:null_dataset).nullify.select_hash_groups(:a, :d).must_equal({})
   end
 end
 
@@ -1307,6 +1423,8 @@ describe "Sequel::Dataset DSL support" do
   
   cspecify "should work with bitwise shift operators", :derby do
     @ds.insert(3, 2)
+    b = Sequel[:b]
+    b = b.cast(:integer) if @db.database_type == :postgres
     @ds.get{a.sql_number << b}.to_i.must_equal 12
     @ds.get{a.sql_number >> b}.to_i.must_equal 0
     @ds.get{a.sql_number << b << 1}.to_i.must_equal 24
@@ -1511,7 +1629,7 @@ describe "Sequel::Dataset DSL support" do
     @ds.insert(20, 10)
     @ds.filter(:a=>20, :b=>10).all.must_equal [{:a=>20, :b=>10}]
     @ds.filter([[:a, 20], [:b, 10]]).all.must_equal [{:a=>20, :b=>10}]
-    @ds.filter({:a=>20}, {:b=>10}).all.must_equal [{:a=>20, :b=>10}]
+    @ds.filter(Sequel.&({:a=>20}, {:b=>10})).all.must_equal [{:a=>20, :b=>10}]
     @ds.filter(Sequel.|({:a=>20}, {:b=>5})).all.must_equal [{:a=>20, :b=>10}]
     @ds.filter(Sequel.~(:a=>10)).all.must_equal [{:a=>20, :b=>10}]
   end
@@ -1545,8 +1663,10 @@ describe "Dataset string methods" do
     @db = DB
     csc = {}
     cic = {}
-    csc[:collate] = @db.dataset_class::CASE_SENSITIVE_COLLATION if defined? @db.dataset_class::CASE_SENSITIVE_COLLATION
-    cic[:collate] = @db.dataset_class::CASE_INSENSITIVE_COLLATION if defined? @db.dataset_class::CASE_INSENSITIVE_COLLATION
+    if @db.database_type == :mssql
+      csc[:collate] = 'Latin1_General_CS_AS'
+      cic[:collate] = 'Latin1_General_CI_AS'
+    end
     @db.create_table!(:a) do
       String :a, csc
       String :b, cic
@@ -1645,6 +1765,27 @@ describe "Dataset string methods" do
     @ds.filter(Sequel.expr(:b).ilike(@ds.escape_like('[Bar%]'))).select_order_map(:b).must_equal ['[bar%]']
     @ds.filter(Sequel.expr(:b).ilike("#{@ds.escape_like('Bar%')}_")).select_order_map(:b).must_equal ['bar%.']
     @ds.filter(Sequel.expr(:b).ilike("#{@ds.escape_like('Bar%')}%")).select_order_map(:b).must_equal ['bar%', 'bar%.', 'bar%..']
+
+    Sequel.extension(:escaped_like)
+    @ds.filter(Sequel.expr(:a).escaped_like('?', 'Foo_')).select_order_map(:a).must_equal []
+    @ds.filter(Sequel.expr(:a).escaped_like('?', 'foo_')).select_order_map(:a).must_equal ['foo_']
+    @ds.filter(Sequel.expr(:b).escaped_like('?', ['bar%'])).select_order_map(:b).must_equal ['bar%']
+    @ds.filter(Sequel.expr(:a).escaped_like('??', ['fo', 'o\\_'])).select_order_map(:a).must_equal ['foo\\_']
+    @ds.filter(Sequel.expr(:b).escaped_like('?', 'bar\\%')).select_order_map(:b).must_equal ['bar\\%']
+    @ds.filter(Sequel.expr(:a).escaped_like('?', '[f#*?oo_]')).select_order_map(:a).must_equal ['[f#*?oo_]']
+    @ds.filter(Sequel.expr(:b).escaped_like('?', '[bar%]')).select_order_map(:b).must_equal ['[bar%]']
+    @ds.filter(Sequel.expr(:b).escaped_like('?_', 'bar%')).select_order_map(:b).must_equal ['bar%.']
+    @ds.filter(Sequel.expr(:b).escaped_like('?%', 'bar%')).select_order_map(:b).must_equal ['bar%', 'bar%.', 'bar%..']
+
+    @ds.filter(Sequel.expr(:a).escaped_ilike('?', 'Foo_')).select_order_map(:a).must_equal ['foo_']
+    @ds.filter(Sequel.expr(:a).escaped_ilike('?', 'Foo_')).select_order_map(:a).must_equal ['foo_']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?', ['Bar%'])).select_order_map(:b).must_equal ['bar%']
+    @ds.filter(Sequel.expr(:a).escaped_ilike('??', ['Fo', 'o\\_'])).select_order_map(:a).must_equal ['foo\\_']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?', 'Bar\\%')).select_order_map(:b).must_equal ['bar\\%']
+    @ds.filter(Sequel.expr(:a).escaped_ilike('?', '[F#*?oo_]')).select_order_map(:a).must_equal ['[f#*?oo_]']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?', '[Bar%]')).select_order_map(:b).must_equal ['[bar%]']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?_', 'Bar%')).select_order_map(:b).must_equal ['bar%.']
+    @ds.filter(Sequel.expr(:b).escaped_ilike('?%', 'Bar%')).select_order_map(:b).must_equal ['bar%', 'bar%.', 'bar%..']
   end
   
   if DB.dataset.supports_regexp?
@@ -1725,34 +1866,6 @@ describe "Dataset identifier methods" do
   end
 end if IDENTIFIER_MANGLING
 
-describe "Dataset defaults and overrides" do
-  before(:all) do
-    @db = DB
-    @db.create_table!(:a){Integer :a}
-    @ds = @db[:a].order(:a).extension(:set_overrides)
-  end
-  before do
-    @ds.delete
-  end
-  after(:all) do
-    @db.drop_table?(:a)
-  end
-  
-  it "#set_defaults should set defaults that can be overridden" do
-    @ds = @ds.set_defaults(:a=>10)
-    @ds.insert
-    @ds.insert(:a=>20)
-    @ds.all.must_equal [{:a=>10}, {:a=>20}]
-  end
-  
-  it "#set_overrides should set defaults that cannot be overridden" do
-    @ds = @ds.set_overrides(:a=>10)
-    @ds.insert
-    @ds.insert(:a=>20)
-    @ds.all.must_equal [{:a=>10}, {:a=>10}]
-  end
-end
-
 if DB.dataset.supports_modifying_joins?
   describe "Modifying joined datasets" do
     before do
@@ -1847,7 +1960,7 @@ describe "Dataset replace" do
   end
 
   it "should update a record if the condition is met" do
-    @d << {:id => 111}
+    @d.insert(:id => 111)
     @d.all.must_equal [{:id => 111, :value => nil}]
     @d.replace(:id => 111, :value => 333)
     @d.all.must_equal [{:id => 111, :value => 333}]
